'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const vscode_languageserver_1 = require("vscode-languageserver");
const vscode_languageserver_2 = require("vscode-languageserver");
const beancount_cli_1 = require("./beancount-cli");
const url = require("url");
const autocomplete_1 = require("./autocomplete");
// Default value for when the file hasn't been loaded yet
//TODO: should we have one of these per open document?
let completionOracle = new autocomplete_1.CompletionOracle(beancount_cli_1.BeancountSymbols.Empty);
const connection = vscode_languageserver_1.createConnection(new vscode_languageserver_1.IPCMessageReader(process), new vscode_languageserver_1.IPCMessageWriter(process));
// Returns the line of the document that contains the specified position
function getFullLine(document, position) {
    const nextLinePosition = { character: 0, line: position.line + 1 };
    const lineOffset = document.offsetAt(position) - position.character;
    const nextLineOffset = document.offsetAt(nextLinePosition);
    return document.getText().substring(lineOffset, nextLineOffset);
}
connection.onCompletion((textDocumentPosition) => {
    connection.console.log("Completion requested");
    const document = documents.get(textDocumentPosition.textDocument.uri);
    const position = textDocumentPosition.position;
    const currentLine = getFullLine(document, position);
    const completions = completionOracle.complete(currentLine);
    const result = [];
    completions.forEach((completion, index) => {
        result.push({
            label: completion.label,
            insertText: completion.insertText,
            kind: vscode_languageserver_2.CompletionItemKind.Text,
            data: index
        });
    });
    return result;
});
const documents = new vscode_languageserver_1.TextDocuments();
function pathToFile(document) {
    return decodeURI(url.parse(document.uri).path);
}
function updateCompletions(document) {
    // Sometimes we don't have real files
    // An example is git diffs. They compare an actual file
    // with an in memory representation of the latest committed
    // file. They have an inmemory:// uri schema.
    // The ledger binary can only point to actual files so the best
    // we can do here is to skip validation for the old version
    if (document.uri.startsWith("file://")) {
        const path = pathToFile(document);
        connection.console.log("Symbols for " + path);
        const symbols = beancount_cli_1.loadSymbols(path);
        connection.console.log("Symbols loaded");
        completionOracle = new autocomplete_1.CompletionOracle(symbols);
    }
}
documents.onDidOpen(params => {
    updateCompletions(params.document);
});
documents.onDidSave((change) => {
    updateCompletions(change.document);
});
documents.listen(connection);
connection.onInitialize((params) => {
    return {
        capabilities: {
            //TODO: evaluate if this should be done incrementally
            textDocumentSync: documents.syncKind,
            completionProvider: {
                resolveProvider: false
            }
        }
    };
});
connection.listen();
//# sourceMappingURL=server.js.map